ma_new_chroot() {
	if ! [ "$(id -u)" = "0" ]
	then
		echo root required.
		return 2
	fi
	# Could also be asked interactively if necessary
	ma_gen i386
}

ma_gen() {
	chr="$MDPC_EXTERNAL_DIR/chroots/$1"
	echo Creating chroot at $chr
	debootstrap --arch=$1 stable "$chr" http://ftp.de.debian.org/debian/ \
								|| exit $?
	cp $MDPC_SCRIPT_DIR/helper/template_chroot_bashrc.sh "$chr/root/.bashrc"
	echo "proc	/proc	proc	defaults,noauto	0	0" > \
								"$chr/etc/fstab"
	chmod 755  "$chr/root/.bashrc"
	chown root "$chr/root/.bashrc"
	chgrp root "$chr/root/.bashrc"
	mkdir "$chr/media/masysmadebpkg"
	chroot "$chr" adduser --disabled-password --gecos "" linux-fan
	cp "$MDPC_SCRIPT_DIR/helper/template_chroot_bashrc.sh" \
						"$chr/home/linux-fan/.bashrc"
	cp "$MDPC_SCRIPT_DIR/helper/template_chroot_bashrc.sh" \
							"$chr/root/.bashrc"
	chmod 755 "$chr/home/linux-fan/.bashrc"
	echo "deb http://192.168.1.16/mdvl/ $MDPC_DIST main contrib non-free" \
						>> "$chr/etc/apt/sources.list"
	chroot "$chr" aptitude update
	chroot "$chr" aptitude -y install devscripts mdvl-dev-libs locales
	chroot "$chr" dpkg-reconfigure locales
}

ma_new_repo() {
	mkdir "$MDPC_REPO"
	mkdir "$MDPC_REPO/conf"
	mkdir "$MDPC_REPO/incoming"
	cp "$MDPC_SCRIPT_DIR/res/gpgkey_exa.dat" "$MDPC_REPO/pubkey"
	cp "$MDPC_SCRIPT_DIR/helpers/transform_into_mdvl.sh" "$MDPC_REPO"
	echo "Origin: Linux-Fan, Ma_Sys.ma
Label: Ma_Sys.ma MDVL Repository
Suite: stable
Codename: $MDPC_DIST
Version: 6.0.6
Architectures: $MDPC_ARCHITECTURES source
SignWith: yes
Components: main non-free contrib
Description: Contains all MDVL-specific packages including configuration files." > $MDPC_REPO/conf/distributions
}

ma_new_dirs() {
	mkdir "$MDPC_SECTION_DIR"
	for i in $MDPC_SECTIONS
	do
		mkdir "$MDPC_SECTION_DIR/$i"
		mf="$MDPC_SECTION_DIR/$i/Makefile"
		echo "# Automatically generated Makefile by mdpc.sh." > "$mf"
		echo                                                 >> "$mf"
		echo "all:"                                          >> "$mf"
		printf "\t$MDPC_RELATIVE_SCRIPT compile $i\n\n"      >> "$mf"
		echo ".PHONY: new"                                   >> "$mf"
		echo "new:"                                          >> "$mf"
		printf "\t$MDPC_RELATIVE_SCRIPT new $i\n\n"          >> "$mf"
	done
	mkdir "$MDPC_EXTERNAL_DIR"
	mkdir "$MDPC_EXTERNAL_DIR/chroots"
}

# Create new package for the section given in $1
ma_new_section() {
	case $1 in
		(debuild) ma_new_debuild;;
		(meta)    ma_new_raw meta .;;
		(raw)     ma_new_raw raw .;;
		(arch)    ma_new_raw arch "$MDPC_ARCHITECTURES";;
		(foreign)
			echo Create a directory below $MDPC_SECTIOND_DIR/foreign
			echo named after your package and put all .deb files
			echo there.
			echo
			echo A foreign package can not be created with this
			echo script.
			return 1;;
		(*)
			echo Unknown section: $1.
			return 1;;
	esac
}

# $1 meta or raw
# $2 destination directories (usually .)
ma_new_raw() {
	printf "Enter package name (without mdvl- prefix): "
	read ma_pkgname
	ma_pkgname=mdvl-$ma_pkgname
	for i in $2
	do
		d="$MDPC_SECTION_DIR/$1/$ma_pkgname/$i"
		mkdir -p "$d" || return 1
		mkdir "$d/DEBIAN"
		arch=all
		if [ "$i" != "." ]
		then
			arch=$i
		fi
		echo "Package: $ma_pkgname
Maintainer: Linux-Fan <Ma_Sys.ma@web.de>
Version: 1.0.0
Section: admin
Priority: optional
Architecture: $arch
Depends: ...
Description: short-desc
 mid-desc" > "$d/DEBIAN/control"
		chmod +x "$d/DEBIAN/control"
	done
}

ma_new_debuild() {
	#
	# Interactive Debian Package creation tool 1.0.0.2,
	# Copyright (c) 2013 Ma_Sys.ma.
	# For further info send an e-mail to Ma_Sys.ma@web.de.
	#
	printf "Enter folder name: "
	read ma_root
	cd "$MDPC_SECTION_DIR/debuild"
	mkdir $ma_root
	cd $ma_root
	printf "Enter package name (without mdvl-, without version): "
	read ma_pkgname
	printf "Enter version (a.b.c~rd): "
	read ma_pkgversion

	ma_pkgname=mdvl-$ma_pkgname
	sub=$ma_pkgname-$ma_pkgversion
	mkdir $sub
	mkdir $sub/debian
	echo 7 > $sub/debian/compat
printf "Enter section [admin|devel|doc|editors|electronics|embedded|games|graphics|httpd|interpreters|java|libs|math|misc|net|science|shells|tex|text|utils|web|x11]: "
	read ma_section
	echo "Source: $ma_pkgname
Maintainer: Linux-Fan <Ma_Sys.ma@web.de>
Section: $ma_section
Priority: optional
Standards-Version: 3.9.1
Build-Depends: debhelper (>= 8)

Package: $ma_pkgname
Architecture: any
Depends: \${shlibs:Depends}, \${misc:Depends}
Description: short-desc
 mid-desc" > $sub/debian/control
	echo "#!/usr/bin/make -f
%:
	dh \$@" > $sub/debian/rules
	chmod +x $sub/debian/rules

	cd $sub
	dch --create -v $ma_pkgversion-1 $ma_pkgname
	cd ..

	mkdir $sub/debian/source
	echo "3.0 (native)" > $sub/debian/source/format

	echo "# Automatically generated makefile, Copyright (c) 2013 Ma_Sys.ma.
# For further info send an e-mail to Ma_Sys.ma@web.de.

all:
	:

install:
	[ -d \$(DESTDIR)/usr/share/mdvl ] || mkdir -p \$(DESTDIR)/usr/share/mdvl

uninstall:
	:" > $sub/Makefile

	echo "Format: http://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
Files: *
Copyright: Copyright (c) 2013 Ma_Sys.ma <Ma_Sys.ma@web.de>
License: GPL-3
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 .
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 .
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 .
 On Debian systems, the full text of the GNU General Public License version 3
 can be found in the file /usr/share/common-licenses/GPL-3" > \
		 $sub/debian/copyright
}
