ma_pkgctrl_section="*"
ma_pkgctrl_packages="*"
ma_pkgctrl_arch="*"
ma_pkgctrl_parsed=false

# $1        Ignored
# $2 to $4  section package arch
# @return 1 upon failure
ma_pkgctrl_parse_args() {
	$ma_pkgctrl_parsed && return 0
	ma_pkgctrl_parsed=true
	shift 2
	[ -z "$1" ] && return 0 # Empty Arg => Abort
	for i in $MDPC_SECTIONS
	do
		if [ "$i" = "$1" ]
		then
			ma_pkgctrl_section=$i
			shift
			break
		fi
	done
	shifted=false
	[ -z "$1" ] && return 0 # Empty Arg => Abort
	for i in $MDPC_ARCHITECTURES
	do
		if [ "$i" = "$1" ]
		then
			ma_pkgctrl_arch=$i
			shift
			shifted=true
			break
		fi
	done
	[ -z "$1" ] && return 0 # Empty Arg => Abort
	if ! $shifted
	then
		ma_pkgctrl_packages="$1"
		shift
		# Nonempty arg => Architecture
		if [ -n "$1" ]
		then
			ma_pkgctrl_arch="$1"
		fi
	fi
}

# $1 action to be performed without prefix
ma_pkgctrl_iterate() {
	for csection in "$MDPC_SECTION_DIR"/$ma_pkgctrl_section
	do
		ma_loop_skip "$csection" && continue
		secname="$(basename "$csection")"
		for cpkg in "$csection"/$ma_pkgctrl_packages
		do
			ma_loop_skip "$cpkg" && continue
			ma_pkgctrl_$1 "$secname" "$(dirname "$cpkg")" \
					"$(basename "$cpkg")" || return $?
		done
	done
}

# @return 1 if $1 = * or non-directory
ma_loop_skip() {
	if [ "$1" = "*" -o ! -d "$1" ]
	then
		return 0
	else
		return 1
	fi
}

# $1 section name
# $2 package directory name
# $3 package name
ma_pkgctrl_update() {
	ma_pkgctrl_update_only "$@" && ma_pkgctrl_compile "$@" && \
						ma_pkgctrl_sync "$@"
}

# $1 section, $2 package directory, $3 package name
ma_pkgctrl_update_only() {
	case "$1" in
		(debuild)
			bakdir="$(pwd)"
			ma_debuild_cd "$@"
			dch -i
			ret=$?
			cd "$bakdir"
			return $ret
			;;
		(meta)  ma_vinc "$2/$3";;
		(raw)   ma_vinc "$2/$3";;
		(arch)  ma_vinc "$2/$3"/*;;
		(foreign)
			echo WARNING: Version of foreign package $3
			echo can not be increased.
			;;
	esac
	return $?
}

ma_debuild_cd() {
	cd "$2/$3"
	for i in *
	do
		ma_loop_skip "$i" && continue
		cd "$i"
		break
	done
}

# $@ directories of raw packages to increment versions.
ma_vinc() {
	while [ "$1" != "" ]
	do
		ma_vincsub "$1/DEBIAN/control" || return $?
		shift
	done
}

# $1 Debian control file to change
ma_vincsub() {
	if ! [ -f "$1" ]
	then
		echo ERROR: $1 does not exist.
		return 1
	fi
	current=$(grep -E "^Version: .*$" "$1" | cut -c9-)
	if [ "$current" = "" ]
	then
		echo Error: $1 does not have a version.
		return 1
	fi
	next=$(echo $current | cut -d. -f -2).$(($(echo $current | \
							cut -d . -f 3) + 1))
	grep -vE "^Version: .*$" "$1" > "$ma_tmp" || return $?
	echo "Version: $next" > "$1" && cat "$ma_tmp" >> "$1" && rm "$ma_tmp"
	return $?
}

ma_pkgctrl_compile() {
	case "$1" in
		(debuild)
			bakdir="$(pwd)"
			ma_debuild_cd "$@"
			debuild -j8 -us -uc
			ret=$?
			if [ "$ret" = "0" ]
			then
				debuild clean
				cd ..
				mv *.deb ..
				# Delete unnecessary files (not very pretty...)
				rm *.debian.tar.gz *.dsc *.build *.changes
			fi
			cd "$bakdir"
			return $ret
			;;
		(meta)  ma_pkgctrl_compile_raw "$2/$3" "$2" false || return $?;;
		(raw)   ma_pkgctrl_compile_raw "$2/$3" "$2" true || return $?;;
		(arch) 
			# Just applied once, persists!
			if [ "$ma_pkgctrl_arch" = "*" ]
			then
				ma_pkgctrl_arch="$MDPC_ARCHITECTURES"
			fi
			ma_pkgctrl_special_arch "$3" "$ma_pkgctrl_arch" || \
								return $?
			for arch in $ma_pkgctrl_arch
			do
				ma_pkgctrl_compile_raw "$2/$3/$arch" "$2" true \
								|| return $?
			done
			;;
		(foreign) cp "$2/$3"/*.deb "$2";;
	esac
}

# $1 src
# $2 destination
# $3 true: calculate size, false: do not calculate size
ma_pkgctrl_compile_raw() {
	if $3
	then
		if ! [ -f "$1"/DEBIAN/control ]
		then
			echo ERROR: $1/DEBIAN/control does not exit.
			return 1
		fi
		echo "Installed-Size: $(du -sk $1/ | cut -f1)" > $ma_tmp
		grep -vF "Installed-Size:" "$1/DEBIAN/control" >> $ma_tmp
		cp $ma_tmp "$1/DEBIAN/control" && rm $ma_tmp
	fi
	fakeroot -- "$0" -f "$1" "$2"
}

# $1 src
# $2 destination
ma_pkgctrl_compile_fakeroot() {
	# A bit problematic / there should be a better way to archive correct
	# file permissions.
	chown -R root "$1" && chgrp -R root "$1" && chmod -R go-w "$1" && \
							dpkg-deb -b "$1" "$2"
	return $?
}

# $1 package name
# $2 arch (can be one or all)
ma_pkgctrl_special_arch() {
	if [ "$1" = "mdvl-dxx-rebirth" ]
	then
		if [ "$2" = "$MDPC_ARCHITECTURES" ]
		then
			echo WARNING: Cannot cross-compile dxx-rebirth.
		else
			bakdir="$(pwd)"
			cd "$MDPC_SECTION_DIR/special/dxx-rebirth"
			./dxx-rebirth.sh "$2"
			ret=$?
			cd "$bakdir"
			return $ret
		fi
	fi
	return $?
}

ma_pkgctrl_sync() {
	match=
	if [ "$1" != "foreign" ]
	then
		match="$3_"
	fi
	#for i in $2/$3_*.deb
	pattern="$2/$match*.deb"
	for i in $pattern
	do
		if [ "$i" = "$pattern" ]
		then
			break
		fi
		carch="$(basename "$i" | cut -d _ -f 3 | cut -d . -f 1)"
		ap=
		if [ "$carch" != "all" ]
		then
			ap=" -A $carch"
		fi
		echo "Synchronizing $3 ($carch)..."
		count=$(reprepro$ap -b $MDPC_REPO list $MDPC_DIST $3 | wc -l)
		if ! [ "$count" = "0" ]
		then
			reprepro$ap -b $MDPC_REPO remove $MDPC_DIST $3 \
							> /dev/null || return $?
		fi
		reprepro -b $MDPC_REPO includedeb $MDPC_DIST $i > /dev/null || \
								return $?
		rm "$i" || return $?
	done
}
